// kaldifst/csrc/matrix/kaldi-matrix.h
#include "kaldifst/csrc/matrix/kaldi-matrix.h"

namespace kaldifst {

template <typename T>
class Matrix {};
template class Matrix<float>;
template class Matrix<double>;

}  // namespace kaldifst
