// kaldifst/csrc/matrix/kaldi-matrix.h

#ifndef KALDIFST_CSRC_MATRIX_KALDI_MATRIX_H_
#define KALDIFST_CSRC_MATRIX_KALDI_MATRIX_H_

namespace kaldifst {

template <typename T>
class Matrix {
 public:
  void Resize(...) {}
};

}  // namespace kaldifst

#endif  // KALDIFST_CSRC_MATRIX_KALDI_MATRIX_H_
